using Asp.Versioning;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Classes.v1.OutGoing;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class OfflineDataController : ControllerBase
{
    [HttpGet]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            List<string> bbb = new();
            foreach (byte[]? blob in row.Data)
            {
                bbb.Add(Convert.ToBase64String(blob));
            }
            Tables.Storage.Update(Storage.ID, SID, Storage.Data.CreateParameter(Array.Empty<byte[]>()));
            return this.ResponseToResult(new OfflineDataBlob()
            {
                data = bbb.ToArray()
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpGet]
    [Route(Luski.Info.Routes.Default.Base + "/Info")]
    public IActionResult GetInfo([FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            bool update = false;
            if (Tables.Sessions.Read(Sessions.StorageID, Sessions.ID.CreateParameter(SSID)) == 0)
            {
                SessionsRow[] rows = Tables.Sessions.ReadRows(uint.MaxValue, Sessions.ID.CreateParameter(SSID), Sessions.StorageID.CreateParameter(SID));
                if (rows.Length == 0)
                {
                    update = true;
                }
                Tables.Sessions.Update(Sessions.StorageID, SID, Sessions.ID.CreateParameter(SSID));
            }
            return this.ResponseToResult(new StorageInfo()
            {
                password = Convert.ToBase64String(row.Password),
                id = row.ID,
                update = update
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(Luski.Info.Routes.Default.Base + "/Info")]
    public IActionResult SetInfo([FromBody]StorageInfo si, [FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Storage.Update(Storage.ID, SID, Storage.Password.CreateParameter(Convert.FromBase64String(si.password)));
            
            return this.ResponseToResult(new StorageInfo()
            {
                password = si.password,
                id = row.ID,
                update = false
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [Route(Luski.Info.Routes.Default.Base + "/Info")]
    public IActionResult PostInfo([FromBody]StorageInfo si, [FromHeader(Name = "token")]string? token)
    {
        try
        {
            Console.WriteLine("POST");
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            Luski.Snowflake sf = Luski.Snowflake.GenerateSnowflake(Luski.Config.ServerEpoch);
            Tables.Storage.Insert(
                Storage.ID.CreateParameter(sf.ID),
                Storage.Password.CreateParameter(Convert.FromBase64String(si.password)),
                Storage.AccountID.CreateParameter(UID),
                Storage.Data.CreateParameter(Array.Empty<byte[]>()),
                Storage.OffileKey.CreateParameter(0));
            Tables.Sessions.Update(Sessions.StorageID, sf.ID, Sessions.ID.CreateParameter(SSID));
            
            return this.ResponseToResult(new StorageInfo()
            {
                password = si.password,
                id = sf.ID,
                update = false
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}