using Asp.Versioning;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Classes.WebTypes;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketBulkMessageController : ControllerBase
{
    [HttpGet]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Get()
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Request.Headers.ContainsKey("id")) return this.ShowError(ErrorCode.MissingHeader, "You did not give an channel id for the channel you want to get a message from");
            string channel_id = Request.Headers["id"].First();
            long MessagesToDownload = 50;
            long Max = 200;
            if (long.TryParse(channel_id, out long Channel_Id) && !Tables.Channels.TryRead(Channels.ID, out _, Channels.ID.CreateParameter(Channel_Id))) return this.ShowError(ErrorCode.InvalidHeader, "The the channel id you have given the server is not a valed id");
            if (!Luski.HasAccessToChannel(ID, Channel_Id, ServerPermissions.ReadMessageHistory))
                return this.ResponseCodeToResult(ErrorCode.Forbidden);
            if (Request.Headers.ContainsKey("messages")) _ = long.TryParse(Request.Headers["messages"].First(), out MessagesToDownload);
            if (MessagesToDownload > Max) MessagesToDownload = Max;
            string? MostRecentID = null;
            string sign = "<";
            if (!Request.Headers.ContainsKey("mostrecentid") || !long.TryParse(MostRecentID = Request.Headers["mostrecentid"].First(), out long mrid) || !Tables.Messages.TryRead(Messages.ID, out _, Messages.ID.CreateParameter(mrid), Messages.ChannelID.CreateParameter(Channel_Id)))
            {
                try
                {
                    bool HasMRID = Tables.Messages.TryRead(Messages.ID, Messages.TimeStamp.GetDecendingOrder(), out long MRID, Messages.ChannelID.CreateParameter(Channel_Id));
                    Console.WriteLine(MRID);
                    if (HasMRID)
                    {
                        MostRecentID = MRID.ToString();
                        sign += "=";
                    }
                }
                catch
                {
                    return this.ResponseToResult(
                        new SocketBulkMessage()
                        {
                            messages = Array.Empty<SocketMessage>()
                        });
                }
            }
            SocketBulkMessage all = new();
            List<SocketMessage> mmmm = new List<SocketMessage>();
            if (MostRecentID != null)
            {
                MessageRow[] rawmsgs = Tables.Messages.ReadRows((uint)MessagesToDownload,
                    Messages.TimeStamp.GetDecendingOrder(),
                    Messages.TimeStamp.CreateParameter(new Luski.Snowflake(long.Parse(MostRecentID)).Timestamp, sign),
                    Messages.ChannelID.CreateParameter(Channel_Id));

                

                foreach (MessageRow Row in rawmsgs)
                {
                    SocketMessage temp = new()
                    {
                        context = Convert.ToBase64String(Row.Context),
                        encoder_type = Row.EncoderType,
                        encryption_key = Row.EncryptionKey,
                        channel_id = Row.ChannelID,
                        user_id = Row.AuthorID,
                        id = Row.ID,
                        files = Array.Empty<Classes.WebTypes.File>()
                    };
                    if (Row.Files.Any())
                    {
                        List<Classes.WebTypes.File> list = new();
                        foreach (long b in Row.Files)
                        {
                            FileRow fileraw = Tables.Files.ReadRow(Files.ID.CreateParameter(b));
                            Classes.WebTypes.File file = new()
                            {
                                id = b,
                                name = Convert.ToBase64String(fileraw.Name),
                                size = ulong.Parse(fileraw.Size.ToString()),
                                name_encoder_type = fileraw.NameEncoderType,
                                name_encryption_key = fileraw.NameEncryptionKey,
                                encoder_type = fileraw.EncoderType,
                                encryption_key = fileraw.EncryptionKey,
                                channel = fileraw.Channel
                            };
                            list.Add(file);
                        }

                        temp.files = list.ToArray();
                    }
                    mmmm.Add(temp);
                }
            }

            if (mmmm is not null) all.messages = mmmm.ToArray();
            else all.messages = Array.Empty<SocketMessage>();
            return this.ResponseToResult(all);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
}