using System.Net.Mime;
using Asp.Versioning;
using JacobTechEncryption.Enums;
using LuskiServer.Classes;
using LuskiServer.Classes.ServerComs;
using LuskiServer.Classes.TableDef;
using LuskiServer.Classes.v1.Incoming;
using LuskiServer.Classes.WebTypes;
using LuskiServer.Enums;
using LuskiServer.Enums.ServerComs;
using Microsoft.AspNetCore.Mvc;
using ServerDatabase;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketMessageController : ControllerBase
{
    [HttpGet]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "msg_id")]long MSG_id)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Messages.TryRead(Messages.ChannelID, out long Channel_Id, Messages.ID.CreateParameter(MSG_id)) ||
                !Luski.HasAccessToChannel(ID, Channel_Id, ServerPermissions.ReadMessageHistory)) this.ResponseCodeToResult(ErrorCode.Forbidden);
            MessageRow msgraw = Tables.Messages.ReadRow(Messages.ID.CreateParameter(MSG_id));
            SocketMessage msg = new()
            {
                user_id = msgraw.AuthorID,
                context = Convert.ToBase64String(msgraw.Context),
                id = MSG_id,
                channel_id = Channel_Id
            };
            if (msgraw.Files.Any())
            {
                List<Classes.WebTypes.File> list = new();
                foreach (long b in msgraw.Files)
                {
                    FileRow fileraw = Tables.Files.ReadRow(Files.ID.CreateParameter(b));
                    Classes.WebTypes.File file = new()
                    {
                        id = b,
                        name = Convert.ToBase64String(fileraw.Name),
                        size = ulong.Parse(fileraw.Size.ToString()),
                        name_encoder_type = fileraw.NameEncoderType,
                        name_encryption_key = fileraw.NameEncryptionKey,
                        encoder_type = fileraw.EncoderType,
                        encryption_key = fileraw.EncryptionKey
                    };
                    list.Add(file);
                }

                msg.files = list.ToArray();
            }

            return this.ResponseToResult(msg);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [DisableRequestSizeLimit]
    [Consumes(typeof(ClientSendMessage), MediaTypeNames.Application.Json)]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]ClientSendMessage data)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Luski.HasAccessToChannel(ID, data.ChannelID, ServerPermissions.SendMessages)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Luski.Snowflake Id = Luski.Snowflake.GenerateSnowflake(Tables.Channels.Read(Channels.Epoch, Channels.ID.CreateParameter(data.ChannelID)));
            ChannelType type = Tables.Channels.Read(Channels.Type, Channels.ID.CreateParameter(data.ChannelID));
            MessageEvent MessageEvent = new()
            {
                UserID = ID,
                ChannelID = data.ChannelID,
                Base64Context = data.Base64Context,
                ID = Id.ID
            };
            Tables.Messages.Insert(
                Messages.ChannelID.CreateParameter(data.ChannelID),
                Messages.ID.CreateParameter(Id.ID),
                Messages.AuthorID.CreateParameter(ID),
                Messages.Context.CreateParameter(Convert.FromBase64String(data.Base64Context)),
                Messages.TimeStamp.CreateParameter(Id.Timestamp),
                Messages.Files.CreateParameter(data.Files),
                Messages.EncoderType.CreateParameter(data.Encoding),
                Messages.EncryptionKey.CreateParameter(data.EncryptionKey));
            WSS.SendData(SendType.ID_Group, new ServerEvent()
            {
                Type = DataType.MessageCreate,
                Data = MessageEvent
            }, Luski.GetChannelBroadcastMembers(data.ChannelID));
            Response.StatusCode = 201;
            return null!;
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    //TODO Add Message edit latter
}