using System.Net.Mime;
using Asp.Versioning;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Classes.v1.OutGoing;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketServerController : ControllerBase
{
    /// <summary>
    /// Server Info.
    /// </summary>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(typeof(ServerInfoJson), StatusCodes.Status200OK)]
    [Produces(MediaTypeNames.Application.Json)]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Get()
    {
        ServerRow sr = Tables.Server.ReadRow(Server.ID.CreateParameter(0));
        ServerInfoJson si = new ServerInfoJson()
        {
            wssv4 = (Luski.Config.IPv4_Overode_WSS is null
                ? $"{(Luski.Config.IPv4SecureWSS ? "wss" : "ws")}://{Luski.Config.IPv4WSS}:{Luski.Config.IPv4PortWSS}{Luski.Config.IPv4_URL_WSS}v1"
                : Luski.Config.IPv4_Overode_WSS + "v1"),
            name = sr.Name,
            description = sr.Description,
            owner = sr.Owner
        };
        return this.ResponseToResult(si);
    }

    [HttpGet]
    [Produces(MediaTypeNames.Application.Json)]
    [Route(Luski.Info.Routes.Default.Base + "/servercom")]
    public IActionResult ServerConnect([FromHeader(Name = "key")]string Key)
    {
        byte[] data = Convert.FromBase64String(Key);
        if (!Tables.AltServers.TryReadRow(out var ser, AltServers.Key.CreateParameter(data)))
            return this.ResponseCodeToResult(ErrorCode.Forbidden);
        return null!;
    }
    
    /// <summary>
    /// Returns the icon for the server.
    /// </summary>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [Produces(MediaTypeNames.Image.Jpeg, MediaTypeNames.Image.Gif, MediaTypeNames.Image.Tiff, "image/png")]
    [Route(Luski.Info.Routes.Default.Base + "/Icon")]
    public IActionResult Icon()
    {
        if (Tables.Server.TryRead(Server.Picture, out byte[]? image, Server.ID.CreateParameter(0)))
        {
            return Tables.Server.Read(Server.PictureType, Server.ID.CreateParameter(0)) switch
            {
                PictureType.png => File(image, "image/png"),
                PictureType.jpeg => File(image, "image/jpeg"),
                PictureType.bmp => File(image, "image/bmp"),
                PictureType.gif => File(image, "image/gif"),
                PictureType.ico => File(image, "image/vnd.microsoft.icon"),
                PictureType.svg => File(image, "image/svg+xml"),
                PictureType.tif => File(image, "image/tiff"),
                PictureType.webp => File(image, "image/webp"),
                //should never happen
                _ => File(image, "image/png"),
            };
        }
        else
        {
            return StatusCode(403, new HTTPResponse()
            {
                error = ErrorCode.Forbidden,
                error_message = "the user you have given does not exist"
            });
        }
    }
}