using System.IO.Compression;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;
using JacobTechEncryption.Enums;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Converters.Matrix.Types;
using LuskiServer.Enums;

namespace LuskiServer.Converters.Matrix;

public static class Matrix
{
    public static void ZipJsonChatExpot(Stream Zip, long Channel, Dictionary<string, long> UserMap,
        List<string> IgnoredUsers, EncoderType Encoder, long Encryption, bool remove = true)
    {
        if (Tables.Channels.TryRead(Channels.ID, out _, Channels.ID.CreateParameter(Channel)))
        {
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            Tables.Files.DeleteRow(Files.Channel.CreateParameter(Channel));
            //throw new Exception("Channel Exist");
        }
        
        using ZipArchive archive = new ZipArchive(Zip, ZipArchiveMode.Read, false);
        string internalfolder = archive.Entries[0].FullName;
        ZipArchiveEntry json = archive.GetEntry(internalfolder + "export.json")!;
        MessageExport export = JsonSerializer.Deserialize(json.Open(), MessageExportContext.Default.MessageExport)!;

        DateTime epoch = DateTime.UnixEpoch.AddMilliseconds(export.Messages[0].Origin).Date;

        try
        {
            Tables.Channels.Update(Channels.ID, Channel,
                //Tables.Channels.Insert(
                Channels.ID.CreateParameter(Channel),
                Channels.Parent.CreateParameter(0),
                Channels.Type.CreateParameter(ChannelType.TextAndVoice),
                Channels.Description.CreateParameter(export.RoomTopic.ToDB(Encoder, Encryption)),
                Channels.Name.CreateParameter(export.RoomName.ToDB(Encoder, Encryption)),
                Channels.RoleOverides.CreateParameter(Array.Empty<long>()),
                Channels.UserOverides.CreateParameter(Array.Empty<long>()),
                Channels.Epoch.CreateParameter(epoch),
                Channels.TitleEncryptionKey.CreateParameter(Encryption),
                Channels.DescriptionEncryptionKey.CreateParameter(Encryption),
                Channels.EncryptionKeys.CreateParameter(new long[] { Encryption }),
                Channels.TitleEncoderType.CreateParameter(Encoder),
                Channels.DescriptionEncoderType.CreateParameter(Encoder),
                //Channels.PictureType.CreateParameter(PictureType.none),
                //Channels.Picture.CreateParameter(Array.Empty<byte>()),
                Channels.EncoderTypes.CreateParameter(new[] { Encoder }));


            foreach (Message msg in export.Messages)
            {
                if (msg.Type != "m.room.message")
                    continue;
                if (!UserMap.ContainsKey(msg.Sender))
                {
                    if (IgnoredUsers.Contains(msg.Sender))
                        continue;
                    if (remove)
                    {
                        Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
                        Tables.Files.DeleteRow(Files.Channel.CreateParameter(Channel));
                    }

                    throw new Exception($"User not in list: {msg.Sender}");
                }

                DateTime cur = DateTime.UnixEpoch.AddMilliseconds(msg.Origin);
                if (msg.Context.Type is null)
                    continue;
                Luski.Snowflake sf = Luski.Snowflake.GenerateSnowflake(epoch, cur);
                cur = cur.ToLocalTime();
                switch (msg.Context.Type)
                {
                    default:

                        Tables.Messages.Insert(
                            Messages.ID.CreateParameter(sf.ID),
                            Messages.Context.CreateParameter(msg.Context.Body.ToDB(Encoder, Encryption)),
                            Messages.ChannelID.CreateParameter(Channel),
                            Messages.EncoderType.CreateParameter(Encoder),
                            Messages.Files.CreateParameter(Array.Empty<long>()),
                            Messages.EncryptionKey.CreateParameter(Encryption),
                            Messages.TimeStamp.CreateParameter(sf.Timestamp),
                            Messages.AuthorID.CreateParameter(UserMap[msg.Sender]));
                        break;
                    case "m.image" or "m.video" or "m.audio" or "m.file":
                        int h = cur.Hour;
                        string pm = "AM";
                        if (h > 11)
                        {
                            h = h - 12;
                            if (h == 0) h = 12;
                            pm = "PM";
                        }

                        string hh = h.ToString();
                        string mm = cur.Minute.ToString();
                        string s = cur.Second.ToString();
                        if (s.Length == 1) s = "0" + s;
                        if (mm.Length == 1) mm = "0" + mm;
                        if (hh.Length == 1) hh = "0" + hh;
                        string[] spl = msg.Context.Body.Split('.');
                        string fil =
                            $"{spl[0]}-{cur.Month}-{cur.Day}-{cur.Year} at {h}-{mm}-{s} {pm}.{spl[spl.Length - 1]}";
                        string loc = msg.Context.Type.Split('.')[1] + "s";
                        if (loc == "audios") loc = "audio";
                        if (fil.EndsWith(".1")) fil = fil.Remove(fil.Length - 2, 2);
                        ZipArchiveEntry? f;
                        if ((f = archive.GetEntry(internalfolder + loc + "/" + fil)) is not null)
                        {
                            byte[] d;
                            using(var memoryStream = new MemoryStream())
                            {
                                f.Open().CopyTo(memoryStream);
                                d = memoryStream.ToArray();
                            }

                            Tables.Files.Insert(
                                Files.ID.CreateParameter(sf.ID),
                                Files.Name.CreateParameter(msg.Context.Body.ToDB(Encoder, Encryption)),
                                Files.Data.CreateParameter(d),
                                Files.EncoderType.CreateParameter(Encoder),
                                Files.EncryptionKey.CreateParameter(Encryption),
                                Files.Hash.CreateParameter(JacobTechEncryption.Encryption.Hashing.SHA256(d)),
                                Files.Owner.CreateParameter(UserMap[msg.Sender]),
                                Files.AllowedChannels.CreateParameter(new long[] { Channel }),
                                Files.Public.CreateParameter(true),
                                Files.Size.CreateParameter(d.LongLength),
                                Files.NameEncoderType.CreateParameter(Encoder),
                                Files.NameEncryptionKey.CreateParameter(Encryption));
                            Tables.Messages.Insert(
                                Messages.ID.CreateParameter(sf.ID),
                                Messages.Context.CreateParameter(Array.Empty<byte>()),
                                Messages.ChannelID.CreateParameter(Encryption),
                                Messages.EncoderType.CreateParameter(Encoder),
                                Messages.Files.CreateParameter(new[] { sf.ID }),
                                Messages.EncryptionKey.CreateParameter(Encryption),
                                Messages.TimeStamp.CreateParameter(sf.Timestamp),
                                Messages.AuthorID.CreateParameter(UserMap[msg.Sender]));
                        }
                        break;
                }
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
            if (remove)
            {
                Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
                Tables.Files.DeleteRow(Files.Channel.CreateParameter(Channel));
            }
            throw;
        }
    }
}