using System.Reflection;
using System.Runtime.InteropServices.JavaScript;
using System.Text;
using Asp.Versioning;
using Asp.Versioning.ApiExplorer;
using JacobTechEncryption;
using JacobTechEncryption.Enums;
using LuskiServer;
using LuskiServer.Classes;
using LuskiServer.Classes.ServerComs;
using LuskiServer.Classes.TableDef;
using LuskiServer.Converters.Matrix;
using LuskiServer.Converters.Matrix.Types;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using ServerDatabase;
using ServerDatabase.Utils;
using Swashbuckle.AspNetCore.SwaggerGen;

[assembly: Microsoft.AspNetCore.Mvc.ApiController]

Luski.Config = Luski.GetSettings("/etc/luskiserver/app.json", AppConfigContext.Default.AppConfig, true);
Console.WriteLine($"Server starting with starting epoch of {Luski.Config.ServerEpoch.ToLocalTime()} and current time of {DateTime.UtcNow.ToLocalTime()}");
Luski.Database = new Database(Luski.Config.Address, 
    Luski.Config.Database, 
    Luski.Config.Username, 
    Luski.Config.Password,
    Luski.Config.CustomeName);


Dictionary<string, List<Message>> fff = new();


try
{
    Luski.Database.ExecuteNonQuery($"CREATE DATABASE {Luski.Config.Database} WITH OWNER = {Luski.Config.Username} ENCODING = 'UTF8' CONNECTION LIMIT = -1 IS_TEMPLATE = False;");
    Console.WriteLine("Database Created");
}
catch
{
    Console.WriteLine("Database Found");
}

foreach (PropertyInfo prop in typeof(Tables).GetProperties())
{
    ITable table = (ITable)prop.GetValue(null)!;
    table.DatabaseHandler = Luski.Database;
    Type t = Type.GetType($"LuskiServer.Classes.TableDef.{prop.Name}")!;
    foreach(PropertyInfo prop2 in t.GetProperties())
    {
        table.AddColumn((ITableColumn)prop2.GetValue(null)!, prop2.Name);
    }
    Luski.Database.Tables.Add(table);
}

Luski.Database.RegisterTables();

Dictionary<string, long> AMap = new()
{
    {"@jacobtech:matrix.org", 0},
    {"@rqndomnezz:matrix.org", 0},
    {"@ardillybar:matrix.org", 0},
    {"@themagicalcats:matrix.org", 0},
    {"@jobotnik:matrix.org", 0},
    {"@captainleader:matrix.org", 0},
    {"@kaizenash:matrix.org", 0},
    {"@bubs_cooper:matrix.org", 0},
    {"@quadro:matrix.org", 0},
    {"@tcll:matrix.org", 0},
    {"@eeetile:matrix.org", 0},
    {"@etile:matrix.org", 0},
    {"@tranquillity_:midov.pl", 0}
};
//Matrix.ZipJsonChatExpot(File.OpenRead("/home/jacob/Downloads/matrix - Luski Project Discussion - Chat Export - 2023-08-08T20-10-12.315Z.zip"), 0, AMap, new(), EncoderType.UTF16, 0);

if (!Tables.Server.TryRead(Server.ID, out _, Server.ID.CreateParameter(0)))
{
    Tables.Server.Insert();
}
//Tables.Server.Update(Server.ID, 0, Server.Picture.CreateParameter(File.ReadAllBytes("/home/jacob/Downloads/XBsECOXDZIXMVIxxMNxRfGRo.png")));
//Tables.Channels.Update(Channels.ID, 1, Channels.PictureType.CreateParameter(PictureType.png), Channels.Picture.CreateParameter(File.ReadAllBytes("/home/jacob/Downloads/zLqHooejmsmKzuMLneUSvjRH.png")));
/*
Tables.Channels.Insert(
    Channels.ID.CreateParameter(1),
    Channels.Parent.CreateParameter(0),
    Channels.Type.CreateParameter(ChannelType.TextAndVoice),
    Channels.Description.CreateParameter(Encoding.UTF8.GetBytes("A nother test Chat")),
    Channels.Name.CreateParameter(Encoding.UTF8.GetBytes("Chat 1")),
    Channels.RoleOverides.CreateParameter(Array.Empty<long>()),
    Channels.UserOverides.CreateParameter(Array.Empty<long>()),
    Channels.Epoch.CreateParameter(DateTime.UtcNow.Date),
    Channels.TitleEncryptionKey.CreateParameter(0),
    Channels.DescriptionEncryptionKey.CreateParameter(0),
    Channels.EncryptionKeys.CreateParameter(new long[]{0}),
    Channels.TitleEncoderType.CreateParameter(EncoderType.UTF8),
    Channels.DescriptionEncoderType.CreateParameter(EncoderType.UTF8),
    Channels.PictureType.CreateParameter(PictureType.png),
    Channels.Picture.CreateParameter(File.ReadAllBytes("/home/jacob/Downloads/4ba.png")),
    Channels.EncoderTypes.CreateParameter(new [] { EncoderType.UTF8 }));
Tables.Channels.Insert(
    Channels.ID.CreateParameter(2),
    Channels.Parent.CreateParameter(1),
    Channels.Type.CreateParameter(ChannelType.TextAndVoice),
    Channels.Description.CreateParameter(Encoding.UTF8.GetBytes("A nother test Chat")),
    Channels.Name.CreateParameter(Encoding.UTF8.GetBytes("Chat 2")),
    Channels.RoleOverides.CreateParameter(Array.Empty<long>()),
    Channels.UserOverides.CreateParameter(Array.Empty<long>()),
    Channels.Epoch.CreateParameter(DateTime.UtcNow.Date),
    Channels.TitleEncryptionKey.CreateParameter(0),
    Channels.DescriptionEncryptionKey.CreateParameter(0),
    Channels.EncryptionKeys.CreateParameter(new long[]{0}),
    Channels.TitleEncoderType.CreateParameter(EncoderType.UTF8),
    Channels.DescriptionEncoderType.CreateParameter(EncoderType.UTF8),
    Channels.PictureType.CreateParameter(PictureType.png),
    Channels.Picture.CreateParameter(File.ReadAllBytes("/home/jacob/Downloads/4ba.png")),
    Channels.EncoderTypes.CreateParameter(new [] { EncoderType.UTF8 }));
Tables.Channels.Insert(
    Channels.ID.CreateParameter(3),
    Channels.Parent.CreateParameter(2),
    Channels.Type.CreateParameter(ChannelType.TextAndVoice),
    Channels.Description.CreateParameter(Encoding.UTF8.GetBytes("A nother test Chat")),
    Channels.Name.CreateParameter(Encoding.UTF8.GetBytes("Chat 3")),
    Channels.RoleOverides.CreateParameter(Array.Empty<long>()),
    Channels.UserOverides.CreateParameter(Array.Empty<long>()),
    Channels.Epoch.CreateParameter(DateTime.UtcNow.Date),
    Channels.TitleEncryptionKey.CreateParameter(0),
    Channels.DescriptionEncryptionKey.CreateParameter(0),
    Channels.EncryptionKeys.CreateParameter(new long[]{0}),
    Channels.TitleEncoderType.CreateParameter(EncoderType.UTF8),
    Channels.DescriptionEncoderType.CreateParameter(EncoderType.UTF8),
    Channels.PictureType.CreateParameter(PictureType.png),
    Channels.Picture.CreateParameter(File.ReadAllBytes("/home/jacob/Downloads/4ba.png")),
    Channels.EncoderTypes.CreateParameter(new [] { EncoderType.UTF8 }));
Tables.Categories.Insert(
    Categories.ID.CreateParameter(1),
    Categories.Name.CreateParameter(Encoding.UTF8.GetBytes("Cat lol")),
    Categories.Parent.CreateParameter(0),
    Categories.Description.CreateParameter(
        Encoding.UTF8.GetBytes("Test cat")),
    Categories.RoleOverides.CreateParameter(Array.Empty<long>())
);
Tables.Categories.Insert(
    Categories.ID.CreateParameter(2),
    Categories.Name.CreateParameter(Encoding.UTF8.GetBytes("Cat lol")),
    Categories.Parent.CreateParameter(1),
    Categories.Description.CreateParameter(
        Encoding.UTF8.GetBytes("Test cat")),
    Categories.RoleOverides.CreateParameter(Array.Empty<long>())
);
*/
if (!Luski.Database.VersionsTable.TryRead(Luski.Database.VersionsTable.ID, out _,
        Luski.Database.VersionsTable.ID.CreateParameter(0)))
{
    Luski.Database.VersionsTable.Insert(Luski.Database.VersionsTable.ID.CreateParameter(0));
}
/*
Tables.Roles.Insert(
    Roles.ID.CreateParameter(1),
    Roles.Name.CreateParameter("Mod"),
    Roles.DisplayName.CreateParameter("Moderators"),
    Roles.Color.CreateParameter("56,155,32,255"),
    Roles.Description.CreateParameter("Mods for the server"),
    Roles.ServerPermissions.CreateParameter(new[]
    {
        ServerPermissions.ViewChannels,
        ServerPermissions.ViewCategories,
        ServerPermissions.Nickname,
        ServerPermissions.SendMessages,
        ServerPermissions.SendFiles,
        ServerPermissions.ChannelAndServerPings,
        ServerPermissions.PingSomeone,
        ServerPermissions.ReadMessageHistory,
        ServerPermissions.UseServerCommands,
        ServerPermissions.JoinVoice,
        ServerPermissions.SpeakInVoice,
        ServerPermissions.ViewThis,
        ServerPermissions.Kick,
        ServerPermissions.CreateCategories,
        ServerPermissions.CreateChannels,
        ServerPermissions.DeleteCategories,
        ServerPermissions.DeleteChannels,
        ServerPermissions.EditCategories,
        ServerPermissions.EditChannels,
        ServerPermissions.EditCategoryPermissions,
        ServerPermissions.EditChannelPermissions,
        ServerPermissions.Invite,
        ServerPermissions.ManageMessages,
        ServerPermissions.ManageRoles,
        ServerPermissions.ViewLogs
    })
);
*/

if (!Tables.Roles.TryRead(Roles.ID, out _, Roles.ID.CreateParameter(0)))
{
    Tables.Roles.Insert(
        Roles.ID.CreateParameter(0),
        Roles.Name.CreateParameter("server"),
        Roles.DisplayName.CreateParameter("Members"),
        Roles.Color.CreateParameter("ÿÿÿÿ"),
        Roles.Description.CreateParameter("The default role for the server. Everybody will have this role."),
        Roles.ServerPermissions.CreateParameter(new[]
        {
            ServerPermissions.ViewChannels,
            ServerPermissions.ViewCategories,
            ServerPermissions.Nickname,
            ServerPermissions.SendMessages,
            ServerPermissions.SendFiles,
            ServerPermissions.ChannelPings,
            ServerPermissions.ServerPings,
            ServerPermissions.PingSomeone,
            ServerPermissions.ReadMessageHistory,
            ServerPermissions.UseServerCommands,
            ServerPermissions.JoinVoice,
            ServerPermissions.SpeakInVoice,
            ServerPermissions.ViewThis
        })
    );
}

if (!Tables.Keys.TryRead(Keys.ID, out _, Keys.ID.CreateParameter(0)))
{
    Tables.Keys.Insert(
        Keys.ID.CreateParameter(0),
        Keys.EncryptionType.CreateParameter(EncryptionType.None),
        Keys.KeyData.CreateParameter(Array.Empty<byte>()));
}

if (!Tables.Categories.TryRead(Categories.ID, out _, Categories.ID.CreateParameter(0)))
{
    Tables.Categories.Insert(
        Categories.ID.CreateParameter(0),
        Categories.Name.CreateParameter(Encoding.UTF8.GetBytes("server")),
        Categories.Parent.CreateParameter(-1),
        Categories.Description.CreateParameter(
            Encoding.UTF8.GetBytes("The default category for the server. Everybody will see this category.")),
        Categories.RoleOverides.CreateParameter(Array.Empty<long>())
    );
    Tables.Channels.Insert(
        Channels.ID.CreateParameter(0),
        Channels.Type.CreateParameter(ChannelType.TextAndVoice),
        Channels.Description.CreateParameter(Encoding.UTF8.GetBytes("Default chat for you to use in your new server")),
        Channels.Name.CreateParameter(Encoding.UTF8.GetBytes("Default Channel")),
        Channels.RoleOverides.CreateParameter(Array.Empty<long>()),
        Channels.UserOverides.CreateParameter(Array.Empty<long>()),
        Channels.Epoch.CreateParameter(DateTime.UtcNow.Date),
        Channels.TitleEncryptionKey.CreateParameter(0),
        Channels.Parent.CreateParameter(0),
        Channels.DescriptionEncryptionKey.CreateParameter(0),
        Channels.EncryptionKeys.CreateParameter(new long[]{0}),
        Channels.TitleEncoderType.CreateParameter(EncoderType.UTF8),
        Channels.DescriptionEncoderType.CreateParameter(EncoderType.UTF8),
        Channels.PictureType.CreateParameter(PictureType.none),
        Channels.Picture.CreateParameter(Array.Empty<byte>()),
        Channels.EncoderTypes.CreateParameter(new [] { EncoderType.UTF8 }));
}

Tables.Sessions.DeleteRows();


foreach (AltServerRow Server in Tables.AltServers.ReadRows())
{
    //TODO add code to open a live WSS connection with all other servers on the network
}

WSS.Init(Luski.Config.IPv4WSS, Luski.Config.IPv4PortWSS, Luski.Config.IPv4_URL_WSS!, Luski.Config.IPv4SecureWSS, Luski.Config.IPv6WSS, Luski.Config.IPv6_URL_WSS, Luski.Config.IPv6PortWSS, Luski.Config.IPv6SecureWSS);

WebApplicationBuilder builder = WebApplication.CreateBuilder( args );

// Add services to the container.

builder.Services.AddControllers();
builder.Services.AddProblemDetails();
builder.Services.AddApiVersioning(
                    options =>
                    {
                        // reporting api versions will return the headers
                        // "api-supported-versions" and "api-deprecated-versions"
                        options.ReportApiVersions = true;
                        
                        /*    Sunset example
                        options.Policies.Sunset( 1 )
                                        //.Effective( DateTimeOffset.Now.Subtract( new TimeSpan(9999999) ) )
                                        .Effective( DateTimeOffset.Now.AddDays( 2 ) )
                                        .Link( "policy.html" )
                                            .Title( "Versioning Policy" )
                                            .Type( "text/html" );
                                            */
                    } )
                .AddMvc()
                .AddApiExplorer(
                    options =>
                    {
                        // add the versioned api explorer, which also adds IApiVersionDescriptionProvider service
                        // note: the specified format code will format the version as "'v'major[.minor][-status]"
                        options.GroupNameFormat = "'v'VVV";

                        // note: this option is only necessary when versioning by url segment. the SubstitutionFormat
                        // can also be used to control the format of the API version in route templates
                        options.SubstituteApiVersionInUrl = true;
                    } );
builder.Services.Configure<ApiBehaviorOptions>(options =>
{
    options.InvalidModelStateResponseFactory = actionContext => 
    {
        string content;
        using (StreamReader reader = new(actionContext.HttpContext.Request.Body))
        {
            content = reader.ReadToEndAsync().Result;
        }

        if (string.IsNullOrEmpty(content)) return new ObjectResult(new HTTPResponse()
        {
            error = ErrorCode.MissingPostData,
            error_message = "The post information was missing"
        })
        {
            StatusCode = 403
        };
        return new ObjectResult(new HTTPResponse()
        {
            error = ErrorCode.InvalidPostData,
            error_message = "The provided post information was invalid"
        })
        {
            StatusCode = 403
        };
    };
});
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddTransient<IConfigureOptions<SwaggerGenOptions>, ConfigureSwaggerOptions>();
builder.Services.AddSwaggerGen(
    options =>
    {
        // add a custom operation filter which sets default values
        options.OperationFilter<SwaggerDefaultValues>();

        string fileName = typeof(Program).Assembly.GetName().Name + ".xml";
        string filePath = Path.Combine(AppContext.BaseDirectory, fileName);

        // integrate xml comments
        options.IncludeXmlComments(filePath, true);
    } );

WebApplication app = builder.Build();
// Configure the HTTP request pipeline.

app.UseSwagger();
//app.UseHttpLogging();
app.UseSwaggerUI(
    options =>
    {
        IReadOnlyList<ApiVersionDescription> descriptions = app.DescribeApiVersions();

        // build a swagger endpoint for each discovered API version
        foreach ( ApiVersionDescription description in descriptions )
        {
            string url = $"/swagger/{description.GroupName}/swagger.json";
            string name = description.GroupName.ToUpperInvariant();
            options.SwaggerEndpoint(url, name);
        }
        options.InjectStylesheet("/www/css/SwaggerDark.css");
        options.DocumentTitle = "Luski Server API";
    } );
app.MapGet("/www/css/SwaggerDark.css", async (CancellationToken t) =>
{
    byte[] css = await File.ReadAllBytesAsync("www/css/SwaggerDark.css", t);
    return Results.File(css, "text/css");
}).ExcludeFromDescription();
app.UseHttpsRedirection();
app.UseAuthorization();
app.MapControllers();
app.Run();